#include <avr/io.h>
#include "lcd-routines.h"
#include <util/delay.h>
 
// sendet ein Datenbyte an das LCD
 
void lcd_data(unsigned char temp1)
{
   unsigned char temp2 = temp1;
 
   LCD_PORT |= (1<<LCD_RS);        	// RS auf 1, "Daten"
   LCD_PORT &= ~(1<<LCD_RW); 		// RW auf 0, "schreiben"
	_delay_us(10);

	// oberes Nibble holen
   temp1 &= 0xF0;
   LCD_PORT &= 0x0F;
   LCD_PORT |= temp1;				// setzen
   	_delay_us(5);
   
   lcd_enable();

   _delay_us(50);

 
 	// unteres Nibble holen
   temp2 <<= 4;
   temp2 &= 0xF0;
   LCD_PORT &= 0x0F;
   LCD_PORT |= temp2;               // setzen
   	_delay_us(5);
   
   lcd_enable();
   
   _delay_us(50);
}
 
// sendet einen Befehl an das LCD
 
void lcd_command(unsigned char temp1)
{
   unsigned char temp2 = temp1;
 
   LCD_PORT &= ~(1<<LCD_RS);        // RS auf 0, "Befehl"
   LCD_PORT &= ~(1<<LCD_RW);		// RW auf 0, "schreiben"
	_delay_us(1);

   // oberes Nibble holen
   temp1 &= 0xF0;              // maskieren
   LCD_PORT &= 0x0F;
   LCD_PORT |= temp1;				// setzen   
   	_delay_us(5);
   lcd_enable();

      _delay_us(50);
 
   // unteres Nibble holen und maskieren
   temp2 <<= 4;
   temp2 &= 0xF0;
   LCD_PORT &= 0x0F;
   LCD_PORT |= temp2;               // setzen   
   	_delay_us(5);
   lcd_enable();
   
   _delay_us(50);
}
 
// erzeugt den Enable-Puls
void lcd_enable(void)
{
   // Bei Problemen ggf. Pause gem Datenblatt des LCD Controllers einfgen
   LCD_PORT |= (1<<LCD_EN);
   _delay_us(5);                   // kurze Pause
   // Bei Problemen ggf. Pause gem Datenblatt des LCD Controllers verlngern
   LCD_PORT &= ~(1<<LCD_EN);
   _delay_us(3);

}
 
// Initialisierung: 
// Muss ganz am Anfang des Programms aufgerufen werden.
 
void lcd_init(void)
{
 
	LCD_PORT &= ~(1<<LCD_RS);      // RS auf 0, "Befehl"
	LCD_PORT &= ~(1<<LCD_RW);      // RW auf 0, "schreiben"

   // muss 3mal hintereinander gesendet werden zur Initialisierung
   _delay_ms(20);
   LCD_PORT &= 0x0F;
   LCD_PORT |= (1<<LCD_DB4) | (1<<LCD_DB5);

   lcd_enable();
   _delay_ms(5);
    
   lcd_enable();
   _delay_ms(1);
    
   lcd_enable();
   _delay_ms(1);
 
   // 4 Bit Modus aktivieren 
   LCD_PORT &= 0x0F;
   LCD_PORT |= (1<<LCD_DB5); //0x20; 
   lcd_enable();
   _delay_ms(1);
 
   // 4Bit / 2 Zeilen / 5x8
   lcd_command(0x28);
 
	//  Cursor verschieben / nicht Display
   lcd_command(0x10);

   //  LCD on / Cursor off / blink off
   lcd_command(0x0C);

   // entry mode / increment
   lcd_command(0x06); 
 
   lcd_clear();
   lcd_home();
}
 
// Sendet den Befehl zur Lschung des Displays
 
void lcd_clear(void)
{
   lcd_command(CLEAR_DISPLAY);
   _delay_ms(5);
}
 
// Sendet den Befehl: Cursor Home
 
void lcd_home(void)
{
   lcd_command(CURSOR_HOME);
   _delay_ms(5);
}
 
// setzt den Cursor in Zeile y (1..4) Spalte x (0..15)
 
void set_cursor(uint8_t x, uint8_t y)
{
  uint8_t tmp;
 
  switch (y) {
    case 1: tmp=0x80+0x00+x; break;    // 1. Zeile
    case 2: tmp=0x80+0x40+x; break;    // 2. Zeile
    case 3: tmp=0x80+0x14+x; break;    // 3. Zeile, angepasst fr 4x20
    case 4: tmp=0x80+0x54+x; break;    // 4. Zeile, angepasst fr 4x20
  }
  lcd_command(tmp);
}
 
// Schreibt einen String auf das LCD
 
void lcd_string(char *data)
{
    while(*data) {
        lcd_data(*data);
        data++;
    }
}
